package INSTALL::common;

use POSIX;

# Always ignore Ctrl-C.
$main::SIG{INT} = "IGNORE";
$main::ENV{TERM} = "vt100";

sub generate_error
{
  print STDERR ("Error: ",join("\n       ",@_),"\n");
  return(undef);
}

sub generate_note
{
  print STDERR ("Note: ",join("\n      ",@_),"\n");
  return(undef);
}

sub generate_report
{
  my $header_length = length($_[0]);
  my $separator = "\n";

  foreach(1..$header_length) {
    $separator .= " ";
  }
  
  print STDERR ($_[0],join($separator,@_[1..$#_]),"\n");
  return(undef);
}

sub generate_indent 
{
  ($#_ >= 0) || die "Invalid number of arguments.";

  my ($length) = shift @_;
  my $spacer;

  if ($length > 0) {
    map { $spacer .= " " } (1 .. $length);
  }

  my $result = INSTALL::common::generate_report($spacer,@_);
  return($result);
}

sub extract_filename 
{
  ($#_ == 0) || die "Invalid number of arguments.";

  my ($path) = @_;
  my @path = split(m@/@,$path);
  
  return($path[$#path]);
}

sub extract_parentdir
{
  ($#_ == 0) || die "Invalid number of arguments.";

  my ($path) = @_;
  my @path = split(m@/@,$path);
  
  return(join("/",@path[0..($#path-1)]));
}

sub strip_extension
{
  ($#_ == 0) || die "Invalid number of arguments.";

  my ($path) = @_;
  $path =~ s/\.[^\.]*$//;
  
  return($path);
}

sub usleep 
{
  ($#_ == 0) || die "Invalid number of arguments.";

  my ($sleeptime)=@_;

  select undef,undef,undef,($sleeptime/1000000);
}

########## Meant to be used as a parent waiting for a child process to die.
sub spinner_wait
{
  ($#_ == 0) || ($#_ == -1) || die "Invalid number of arguments.";

  my $child = -1;
  my $pid = -1;

  if ($#_ == 0) {
    ($pid) = @_;
  }

  my @SPINLIST = ("|\b", "/\b", "-\b", "\\ \b\b", "|\b", "/\b", "-\b", "\\ \b\b");
  my $length = $#SPINLIST + 1;
  
  do { 
    $child = waitpid(-1,&WNOHANG);
    $SPINCOUNT = ($SPINCOUNT + 1) % $length;
    print STDERR ("$SPINLIST[$SPINCOUNT]");
    usleep(250000);
  } until(($child == $pid) || ($child == -1));
}

########## Meant to be used as a child process waiting to be signaled by 
########## an outside process (probably the parent).

sub spinner_wait_signal
{
  ($#_ == 0) || ($#_ == -1) || die "Invalid number of arguments.";

  my $child = -1;
  my $pid = -1;

  if ($#_ == 0) {
    ($pid) = @_;
  }

  my @SPINLIST = ("|\b", "/\b", "-\b", "\\ \b\b", "|\b", "/\b", "-\b", "\\ \b\b");
  my $length = $#SPINLIST + 1;
  
  do { 
    $SPINCOUNT = ($SPINCOUNT + 1) % $length;
    print STDERR ("$SPINLIST[$SPINCOUNT]");
    usleep(250000);
  } until(0);  
}

sub signal_spinner
{
  ($#_ == 0) || die "Invalid number of arguments.";
  my ($pid) = @_;

  kill(&SIGTERM,$pid);
  wait;
  print "\n";
}

#------------------------------------------------------------------------------
# This method will establish the globals used universally throughout the 
# installer.  It should be noted that this method assumes that the
# $pParameters->{asphome} value was set prior to execution.
#------------------------------------------------------------------------------
use INSTALL::global;
use INSTALL::parameter;

my $pParameters = \%INSTALL::parameter::variables;

sub set_standard_globals
{
  (exists $pParameters->{asphome}) || die "Required parameter asphome is not set.";
  (-d $pParameters->{asphome}) || die "Required parameter asphome refers to an invalid directory.";

  # Strip trailing / off of asphome directory.
  $pParameters->{asphome} =~ s@/+$@@;

  if ($pParameters->{asphome} =~ /^\s*$/) {
    $pParameters->{asphome} = "/";
  }

  %INSTALL::global::variables =
    ( 
     casp_version   => "3.5.2",
     odbc_version   => "15",
     license_name   => "LICENSE.LIC",
     perl		  => "",
     execute_params => join(" ",map { "\"$_\"" } @main::ARGV),
     status_text    => "",
     asphome        => $pParameters->{asphome},
     asp_dir        => $pParameters->{asphome},
     log_dir        => "$pParameters->{asphome}/logs",
     module_dir     => "$pParameters->{asphome}/module",
     install_dir    => "$pParameters->{asphome}/INSTALL",
     templates_dir  => "$pParameters->{asphome}/INSTALL/templates",
     template_dir   => "$pParameters->{asphome}/INSTALL/templates",
     components_dir => "$pParameters->{asphome}/components",
     component_dir  => "$pParameters->{asphome}/components",
     
     chili_ini_dir  => "/var/opt/casp",
     chili_ini      => "/var/opt/casp/chili.ini",
     installed_db   => "$pParameters->{asphome}/.installed_db",
     settings_reg   => "$pParameters->{asphome}/INSTALL/settings.reg",
     
     install_log    => "$pParameters->{asphome}/logs/install",
     summary_log    => "$pParameters->{asphome}/logs/install_summary",
     register_log   => "$pParameters->{asphome}/logs/register",
     component_log  => "$pParameters->{asphome}/logs/component"
    );

  return(1);
}

#------------------------------------------------------------------------------
# This method returns the realpath of the filename / path specified.  See the
# realpath manpage.  If the file does not exist undef is returned.
#------------------------------------------------------------------------------

sub realpath 
{
  ($#_ == 0) || die "Invalid number of arguments.";
  my ($filename) = @_;    

  # Remove relative path dependency.
  if ($filename =~ m@^[^/]@) {
    # Relative path.
    my $pwd = `pwd`;
    chomp $pwd;

    $filename = "$pwd/$filename";
  }

  if (-f $filename) {
    my $done;
    
    if (-l $filename) {
      my $symlink = readlink($filename);
      
      if ($symlink =~ m@^[^/]@) {
	# Relative path.
	my $parent_dir = $filename;
	$parent_dir =~ s@^(.*)/[^/]*$@$1@;

	return(realpath("$parent_dir/$symlink"));
      }	
      else {
	return(realpath($symlink));
      }
    }
    else {
      return($filename);
    }
  }
  else {
    return(undef);
  }
}

1;
