package INSTALL::os;

$prior_detection = 0;

#INSTALL::os::Details
%Details = 
  ( os => "Unknown",
    ostag => "Unknown",
    perl  => "Unknown",
    hardware => "Unknown",
    release => "Unknown",
    default_runlevel => "Unknown",
    package_formats => "Unknown",
    distribution => "Unknown", 		# Only valid on Linux systems.
    distribution_flavor => "Unknown",   # Only valid on Linux systems.
    distribution_version => "Unknown",	# Only valid on Linux systems.
  );

sub error {
  print STDERR "Error: ",@_;
  return(0);
}

sub detect_os {
  return(`uname -s`);
}

sub detect_hardware {
  my $hardware = `uname -m`;
  
  # Correct for naming conventions.
 HARDWARE: 
  for ($hardware) {
    /i[x3456]86/ && 
      do {
	$hardware = "ia32";
	last HARDWARE;
      };
    #[789] ia64?
  }
  return($hardware);
}

sub detect_release {
  return(`uname -r`);
}

sub detect_default_runlevel {
  open(INITTAB,"</etc/inittab") || die "Unable to open /etc/inittab for reading.";
  my $default_runlevel;
  
  while(<INITTAB>) {
    if (/([0-9]):initdefault/) {
      $default_runlevel = $1;
      last;
    }
  }
  
  close(INITTAB);
  
  ($default_runlevel =~ /[0-9]+/) || die "Unable to determine default run-level.";

  return($default_runlevel);
}

sub detect_perl {
  # scan all the normal places

  my @perl_locations = 
    (
     "/usr/bin/perl",
     "/usr/local/bin/perl",
     "/opt/perl/bin/perl",
     `/usr/bin/which perl`
    );

  my @perl_list = grep { $_ if (-x $_) } @perl_locations;

  if ($#perl_list >= 0) {
    return($perl_list[0]);
  }
  else {
    return("Unknown");
  }
}

# Returns non-zero on success.
sub detect {
  if (!$prior_detection) {
    $Details{os} = detect_os;
    $Details{hardware} = detect_hardware;
    $Details{release}  = detect_release;
    $Details{default_runlevel} = detect_default_runlevel;
    $Details{perl} = detect_perl;
    
    if ($Details{os} =~ /linux/i) {
    DISTRIBUTION:
      while(1) {
	if (-e "/etc/mandrake-release") {
	  $Details{distribution} = "Mandrake";
	  $Details{distribution_flavor} = "RedHat";

	  my $redhat;

	  if (open(MANDRAKE,"</etc/mandrake-release")) {
	    my $line = <MANDRAKE>;
	    $line =~ s/[^0-9]*([0-9]+\.[0-9]+).*/\1/;
	    $mandrake = $line;

	    if ($mandrake =~ /[0-9]+/) {
	      $Details{distribution_version} = $mandrake;
	    }
	  }
	  
	  $Details{package_formats} = "RPM";
	  last DISTRIBUTION;
	}

	if (-e "/etc/redhat-release") {
	  $Details{distribution} = "RedHat";
	  $Details{distribution_flavor} = "RedHat";

	  my $redhat;

	  if (open(REDHAT,"</etc/redhat-release")) {
	    my $line = <REDHAT>;
	    $line =~ s/[^0-9]*([0-9]+\.[0-9]+).*/\1/;
	    $redhat = $line;

	    if ($redhat =~ /[0-9]+/) {
	      $Details{distribution_version} = $redhat;
	    }
	  }
	  
	  $Details{package_formats} = "RPM";
	  last DISTRIBUTION;
	}
	
	if (-e "/etc/cobalt-release") {
	  $Details{distribution} = "Cobalt";
	  $Details{distribution_flavor} = "RedHat";

	  my $cobalt;
	  
	  if (open(COBALT,"</etc/cobalt-release")) {
	    my $line = <COBALT>;
	    $line =~ s/[^0-9]*([0-9]+\.[0-9]+).*/\1/;
	    $cobalt = $line;
	    
	    if ($cobalt =~ /[0-9]+/) {
	      $Details{distribution_version} = $cobalt;
	    }
	  }

	  $Details{package_formats} = "RPM,PKG";
	  last DISTRIBUTION;
	}

	if (-e "/etc/slackware-version") {
	  $Details{distribution} = "Slackware";
	  $Details{distribution_flavor} = "Slackware";

	  my $slackware;
	  
	  if (open(SLACKWARE,"</etc/slackware-version")) {
	    my $line = <SLACKWARE>;
	    $line =~ s/[^0-9]*([0-9]+(\.[0-9]+)*).*/\1/;
	    $slackware = $line;
	    
	    if ($slackware =~ /[0-9]+/) {
	      $Details{distribution_version} = $slackware;
	    }
	  }

	  $Details{package_formats} = "TAR";
	  last DISTRIBUTION;
	}

	if (-e "/etc/SuSE-release") {
	  $Details{distribution} = "SuSE";
	  $Details{distribution_flavor} = "SuSE";

	  my $SuSE;
	  
	  if (open(SUSE,"</etc/SuSE-release")) {
	    
	    while(<SUSE>) {
	      if (/VERSION\s+=\s+(\d.*)/) {
		$SuSE = $1;
	      }
	    }

	    if ($SuSE =~ /[0-9]+/) {
	      $Details{distribution_version} = $SuSE;
	    }
	  }

	  $Details{package_formats} = "RPM";
	  last DISTRIBUTION;	  
	}

	die "Unable to determine distribution.";
      }
    }
    
    foreach (keys %Details) {
      chomp $Details{$_};
    }

    $prior_detection = 1;
  }

  $Details{ostag} = lc($Details{os}) . substr($Details{release},0,1);
  
  return(1);
}

detect;

1;
