# This package is responsible for maintaining a list of global parameter
# variables, which are shared among all executing packages. Basically, 
# by default, this consists of maintaining a centralized hash, named 
# variables, and allowing users to add and delete from this hash at free.
#
# Note: A parameter, by its very essence is a 

package INSTALL::parameter;
%variables;

sub delete {
  map { delete $variables{$_} } @_;
  return(1);
}

sub append {
  my %hash = @_;
  my @keys = keys %hash;

  map { $variables{$_} = $hash{$_} } @keys;
  return(1);
}


sub setup {
  my $i = 0;

  foreach (@_) {
    if (/\s*([^\s=]*)\s*=\s*(.*)/) {
      my $key = $1;
      my $value = $2;
      
      if ($value =~ /\"([^\"]*|\\\"*)\"/) {
	$variables{$key} = $1;
      }
      else {
	$variables{$key} = $value;
      }
    }
    else {
      # Unnamed arguments, use incremental naming scheme.
      $variables{$i++} = $_;
    }
  }  

  return(1);
}

setup(@main::ARGV);
