package INSTALL::reboot;

use INSTALL::global;
use INSTALL::os;
use INSTALL::substitute;

my $pOS = \%INSTALL::os::Details;

my $pGlobals = \%INSTALL::global::variables;
my $CONST_Package = "INSTALL::reboot";

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Internal functions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#------------------------------------------------------------------------------
# All of the error reporting mechanisms take a list of strings, and format
# them such that they fall on different lines.
#------------------------------------------------------------------------------
sub error {
  my $result = INSTALL::common::generate_error(@_);
  print STDERR "Package location: $CONST_Package\n";
  return($result);
}

sub trace
{
  if ($debug)
  {
    my $result = INSTALL::common::generate_report("trace: ",@_);
    print STDERR "Package location: $CONST_Package\n";
    return($result);
  }
}

sub report 
{
  my $result = INSTALL::common::generate_report("",@_);
  print STDERR "Package location: $CONST_Package\n";
  return($result);
}

my $RedHat_RCFile =
'#!/bin/sh
#------------------------------------------------------------------------------
# Generated for Chili!Soft ASP Server
#
# Author: Chili!Soft, Inc. (2000)
# Script: <<reboot::script_name>>
#------------------------------------------------------------------------------

# Source function library.
. /etc/rc.d/init.d/functions

# See how we were called.
case "$1" in
  start)
        echo -n "Executing start script (<<reboot::script_name>>): "
	<<reboot::start_script>> > /dev/null 2>&1 && success || failed $1
        echo
        touch /var/lock/subsys/<<reboot::script_name>> > /dev/null 2>&1
        ;;
  stop)
        echo -n "Executing stop script (<<reboot::script_name>>): "
	<<reboot::stop_script>> > /dev/null 2>&1 && success || failed $1
        echo
        rm -f /var/lock/subsys/<<reboot::script_name>> > /dev/null 2>&1
        ;;
  status)
        echo "Executing status script(<<reboot::script_name>>): "
	<<reboot::status_script>>
        ;;
  restart)
        $0 stop
        $0 start
        ;;
  *)
        echo "Usage: $0 {start|stop|restart|status}"
        exit 1
esac

exit 0
';

my $SuSE_RCFile =
'#! /bin/sh
#------------------------------------------------------------------------------
# Generated for Chili!Soft ASP Server
#
# Author: Chili!Soft, Inc. (2000)
# Script: <<reboot::script_name>>
#------------------------------------------------------------------------------

. /etc/rc.config

# Set this to something other than yes to disable automatic restart on system
# boot.  If it is set to something other than yes, the script will only 
# execute if run by a user (ie. not during runlevel changes).
START_ASP=yes

# Determine the base and follow a runlevel link name.
base=${0##*/}
link=${base#*[SK][0-9][0-9]}

# Force execution if not called by a runlevel directory.
test $link = $base && START_ASP_=yes
test "$START_ASP" = yes || exit 0

# The echo return value for success (defined in /etc/rc.config).
return=$rc_done
case "$1" in
    start)
        echo -n "Executing start script (<<reboot::script_name>>): "
	<<reboot::start_script>> > /dev/null 2>&1 || return=$rc_failed
        echo -e "$return"

        touch /var/lock/subsys/<<reboot::script_name>> > /dev/null 2>&1
        ;;
    stop)
        echo -n "Executing stop script (<<reboot::script_name>>): "
	<<reboot::stop_script>> > /dev/null 2>&1 || return=$rc_failed
        echo -e "$return"

        rm -f /var/lock/subsys/<<reboot::script_name>> > /dev/null 2>&1
        ;;
    restart)
        $0 stop || return=$rc_failed
        $0 start || return=$rc_failed
        ;;
    status)
        echo "Executing status script(<<reboot::script_name>>): "
	<<reboot::status_script>>
        ;;
    *)
        echo "Usage: $0 {start|stop|status|restart}"
        exit 1
        ;;
esac

# Inform the caller not only verbosely and set an exit status.
test "$return" = "$rc_done" || exit 1
exit 0
';

my $Slackware_RCFile = 
'#! /bin/sh
#------------------------------------------------------------------------------
# Generated for Chili!Soft ASP Server
#
# Author: Chili!Soft, Inc. (2000)
# Script: <<reboot::script_name>>
#------------------------------------------------------------------------------

rc_success="Success."
rc_failed="Failure."

return=$rc_success
case "$1" in
    start)
        echo -n "Executing start script (<<reboot::script_name>>): "
	<<reboot::start_script>> > /dev/null 2>&1 || return=$rc_failed
        echo -e "$return"

        touch /var/lock/subsys/<<reboot::script_name>> > /dev/null 2>&1
        ;;
    stop)
        echo -n "Executing stop script (<<reboot::script_name>>): "
	<<reboot::stop_script>> > /dev/null 2>&1 || return=$rc_failed
        echo -e "$return"

        rm -f /var/lock/subsys/<<reboot::script_name>> > /dev/null 2>&1
        ;;
    restart)
        $0 stop || return=$rc_failed
        $0 start || return=$rc_failed
        ;;
    status)
        echo "Executing status script(<<reboot::script_name>>): "
	<<reboot::status_script>>
        ;;
    *)
        echo "Usage: $0 {start|stop|status|restart}"
        exit 1
        ;;
esac

# Inform the caller not only verbosely and set an exit status.
test "$return" = "$rc_success" || exit 1
exit 0
';
	
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# External functions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#------------------------------------------------------------------------------
# This method returns non-undef / true in the case of automatic reboot being
# enabled for the running distribution.
#------------------------------------------------------------------------------
sub supported {
  if (($pOS->{distribution_flavor} eq "RedHat") ||
      ($pOS->{distribution_flavor} eq "SuSE")   ||
      ($pOS->{distribution_flavor} eq "Slackware"))
  {
    return(1);
  }
  else {
    return(undef);
  }
}

sub install_script 
{
  my ($script_name, $start_script, $stop_script, $status_script, $start_priority, $stop_priority) = @_;

  #---------------------------------------------------------------------------
  # RedHat
  #---------------------------------------------------------------------------  
  if ($pOS->{distribution_flavor} =~ /RedHat/i) {
    my $script_path = "/etc/rc.d/init.d/$script_name";

    unlink($script_path);

    my %substitute = 
      (
       reboot::script_name => $script_name,
       reboot::start_script => $start_script,
       reboot::stop_script => $stop_script,
       reboot::status_script => $status_script,
       reboot::start_priority => $start_priority,
       reboot::stop_priority => $stop_priority
      );

    INSTALL::substitute::string_to_file
      ({ map { ("<<$_>>" => $substitute{$_}) } keys %substitute },
       $script_path,
       $RedHat_RCFile
      );

    chmod(0755,$script_path);
    
    my $source = "../init.d/$script_name";

    # Generate start links.
    foreach (3 .. 5) {
      my $destination = "/etc/rc.d/rc$_.d/S$start_priority$script_name";

      unlink($destination);
      unless(symlink($source,$destination)) {
	uninstall_script($script_name,$start_priority,$stop_priority);
	return(error("Unable to make symlink from $source to $destination.\n"));
      }
    }

    # Generate kill links.
    foreach (0, 1, 2, 6) {
      my $destination = "/etc/rc.d/rc$_.d/K$stop_priority$script_name";

      unlink($destination);
      unless(symlink($source,$destination)) {
	uninstall_script($script_name,$start_priority,$stop_priority);
	return(error("Unable to make symlink from $source to $destination.\n"));
      }
    }    

    return(1);
  }
  #---------------------------------------------------------------------------
  # SuSE
  #---------------------------------------------------------------------------  
  elsif ($pOS->{distribution_flavor} =~ /SuSE/i) {
    my $script_path = "/sbin/init.d/$script_name";

    unlink($script_path);
    my %substitute = 
      (
       reboot::script_name => $script_name,
       reboot::start_script => $start_script,
       reboot::stop_script => $stop_script,
       reboot::status_script => $status_script,
       reboot::start_priority => $start_priority,
       reboot::stop_priority => $stop_priority
      );

    INSTALL::substitute::string_to_file
      ({ map { ("<<$_>>" => $substitute{$_}) } keys %substitute },
       $script_path,
       $SuSE_RCFile
      );
    
    chmod(0755,$script_path);

    unless(symlink($script_path,"/sbin/rc$script_name")) {
      uninstall_script($script_name,$start_priority,$stop_priority);
      return(error("Unable to make symlink from $script_path to /sbin/rc$script_name"));
    }
    
    my $source = "../$script_name";

    # Generate start links.
    foreach (2 .. 5) {
      my $destination = "/etc/rc.d/rc$_.d/S$start_priority$script_name";

      unlink($destination);
      unless(symlink($source,$destination)) {
	uninstall_script($script_name,$start_priority,$stop_priority);
	return(error("Unable to make symlink from $source to $destination.\n"));
      }
    }

    # Generate kill links.
    foreach (0, 1, 6) {
      my $destination = "/etc/rc.d/rc$_.d/K$stop_priority$script_name";

      unlink($destination);
      unless(symlink($source,$destination)) {
	uninstall_script($script_name,$start_priority,$stop_priority);
	return(error("Unable to make symlink from $source to $destination.\n"));
      }
    }    

    return(1);
  }
  #---------------------------------------------------------------------------
  # Slackware
  #---------------------------------------------------------------------------  
  elsif ($pOS->{distribution_flavor} =~ /Slackware/i) {
    ### Even though Slackware uses BSD-style init scripts still, they have 
    ### made SysV-style init scripts available through one BSD-style script
    ### which is responsible for checking the existence of any SysV-style 
    ### scripts and executing them (rc.sysvinit).

    my $script_dir = "/etc/rc.d/init.d";
    
    if (not -d $script_dir) {
      unless(mkdir($script_dir,0755)) {
	return(error("Unable to create $script_dir for SysV-style init scripts."));
      }
    }

    my $script_path = "$script_dir/$script_name";

    unlink($script_path);
    my %substitute = 
      (
       reboot::script_name => $script_name,
       reboot::start_script => $start_script,
       reboot::stop_script => $stop_script,
       reboot::status_script => $status_script,
       reboot::start_priority => $start_priority,
       reboot::stop_priority => $stop_priority
      );

    INSTALL::substitute::string_to_file
      ({ map { ("<<$_>>" => $substitute{$_}) } keys %substitute },
       $script_path,
       $Slackware_RCFile
      );
    
    chmod(0755,$script_path);
    
    my $source = "../init.d/$script_name";

    # Generate start links.
    foreach (2 .. 5) {
      my $destination_dir = "/etc/rc.d/rc$_.d";
      
      if (not -d $destination_dir) {
	unless(mkdir($destination_dir,0755)) {
	  uninstall_script($script_name,$start_priority,$stop_priority);
	  return(error("Unable to create directory $destination_dir for SysV-style init script."));
	}
      }

      my $destination = "$destination_dir/S$start_priority$script_name";

      unlink($destination);
      unless(symlink($source,$destination)) {
	uninstall_script($script_name,$start_priority,$stop_priority);
	return(error("Unable to make symlink from $source to $destination.\n"));
      }
    }

    # Generate kill links.
    foreach (0, 1, 6) {
      my $destination_dir = "/etc/rc.d/rc$_.d";

      if (not -d $destination_dir) {
	unless(mkdir($destination_dir,0755)) {
	  uninstall_script($script_name,$start_priority,$stop_priority);
	  return(error("Unable to create directory $destination_dir for SysV-style init script."));
	}
      }

      my $destination = "$destination_dir/K$stop_priority$script_name";

      unlink($destination);
      unless(symlink($source,$destination)) {
	uninstall_script($script_name,$start_priority,$stop_priority);
	return(error("Unable to make symlink from $source to $destination.\n"));
      }
    }    

    return(1);
  }
  #---------------------------------------------------------------------------
  # Non-supported.
  #---------------------------------------------------------------------------  
  else {
    return(undef);
  }
}

sub uninstall_script
{
  my ($script_name,$start_priority, $stop_priority) = @_;

  #---------------------------------------------------------------------------
  # RedHat
  #---------------------------------------------------------------------------  
  if ($pOS->{distribution_flavor} eq "RedHat") {
    unlink("/etc/rc.d/init.d/$script_name");

    foreach (0 .. 6) {
      unlink("/etc/rc.d/rc$_.d/K$stop_priority$script_name");
      unlink("/etc/rc.d/rc$_.d/S$start_priority$script_name");
    }

    return(1);
  }
  #---------------------------------------------------------------------------
  # SuSE
  #---------------------------------------------------------------------------  
  elsif ($pOS->{distribution_flavor} eq "SuSE") {
    unlink("/sbin/init.d/$script_name");
    unlink("/sbin/rc$script_name");

    foreach (0 .. 6) {
      unlink("/etc/rc.d/rc$_.d/K$stop_priority$script_name");
      unlink("/etc/rc.d/rc$_.d/S$start_priority$script_name");
    }

    return(1);
  }
  #---------------------------------------------------------------------------
  # Slackware
  #---------------------------------------------------------------------------  
  elsif ($pOS->{distribution_flavor} eq "Slackware") {
    unlink("/etc/rc.d/init.d/$script_name");

    foreach (0 .. 6) {
      unlink("/etc/rc.d/rc$_.d/K$stop_priority$script_name");
      unlink("/etc/rc.d/rc$_.d/S$start_priority$script_name");
    }

    return(1);
  }
  else {
    return(undef);
  }
}
