
use INSTALL::common;
 
use INSTALL::global;
use INSTALL::parameter;
use INSTALL::os;
use INSTALL::conditional;
use INSTALL::webserver;
use INSTALL::component;
use IO::Handle;
 
umask 0;
 
my $CONST_Package = "upgrade.pl - main";
 
STDOUT->autoflush(1);
 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Internal functions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#------------------------------------------------------------------------------
# All of the error reporting mechanisms take of a list of strings, and format
# them such that they fall on different lines.
#------------------------------------------------------------------------------
sub error {
  my $result = INSTALL::common::generate_error(@_);
  print STDERR "Package location: $CONST_Package\n";
  return($result);
}
 
sub trace
{
  if ($debug)
  {
    my $result = INSTALL::common::generate_report("trace: ",@_);
    print STDERR "Package location: $CONST_Package\n";
    return($result);
  }
}
 
sub report
{
  my $result = INSTALL::common::generate_report("",@_);
  print STDERR "Package location: $CONST_Package\n";
  return($result);
} 

sub status
{
	system("echo @_ >> upgrade.log");
}	

my $pParameters = \%INSTALL::parameter::variables;
 
#==============================================================================
#------------------------ Default global values
#==============================================================================
 
INSTALL::common::set_standard_globals;
 
my $pGlobals = \%INSTALL::global::variables;
my $pOS = \%INSTALL::os::Details;
                                     
sub fatal_error
{
  error(@_);
  exit(1);
}
    
use INSTALL::ini;
use INSTALL::apache;

# check for action parameter
########################################################
if (!exists $pParameters->{action})
{
	fatal_error("Action was not specified");
}
elsif ($pParameters->{action} eq "save")
{
	if (upgrade_save())
	{
		fatal_error("Failed saving installation data");
	}
}
elsif ($pParameters->{action} eq "restore")
{
	if (upgrade_restore())
	{
		fatal_error("Failed restoring installation data");
	}
}
else
{
	fatal_error("Incorrect action was specified - $pParameters->{action}");
}
exit(0);


sub upgrade_save
{



	return(0);
}


sub upgrade_restore
{
	# check parameters
	########################################################
	if (!exists $pParameters->{def_file})
	{
		fatal_error("Definition file was not specified");
	}
	elsif (!-f $pParameters->{def_file})
	{
		fatal_error("Definition file was not found - $pParameters->{def_file}");
	}
	
	if (!exists $pParameters->{asphome})
	{
	        fatal_error("Asphome was not specified");
	}
	elsif (!-d $pParameters->{asphome})
	{
	        fatal_error("Asphome directory was not found - $pParameters->{asphome}");
	}   
	$path = $pParameters->{asphome};
	
	# open definition file
	#########################################################
	$def = INSTALL::ini::load($pParameters->{def_file});
	if (!$def)
	{
		fatal_error("Definition file could not be loaded - $pParameters->{def_file}"); 
	}
	$psec = INSTALL::ini::openSection($def, "upgrade");
	if (!$psec)
	{
		fatal_error("Definition file format is not correct - upgrade section");
	}
	
	# make sure we have the correct definition file version
	#########################################################
	if ($psec->{definition_version} ne "1")
	{
		fatal_error("Incorrect definition file version - $psec->{definition_version}");
	}
	
	# make sure we are upgrading from the correct version
	#########################################################
	if ($psec->{old_version} ne "3.5.01")
	{
		fatal_error("Cannot upgrade from current version - $psec->{old_version}");
	}
	
	# copy the included license file
	#########################################################
	if (-f "$psec->{license_file}")
	{
		`cp $psec->{license_file} $path`;
	}
	                  
	# run initial install script
	#########################################################
	$pwd=`pwd`;
	chomp $pwd;
	chdir($path);
	$ret = system("/usr/bin/perl -I. INSTALL/caspi2.pl \"asphome=$path\" \"non_interactive=yes\" \"install_webserver=no\" \"install_admin=no\"");
	if ($ret)
	{
		fatal_error("Primary installation procedure failed.");
	}

	# install all web servers
	#########################################################
	$server_count = $psec->{servers};
	$i = 0;
	while ($i < $server_count)
	{        
		$server = INSTALL::ini::openSection($def, "server$i");
		if ($server)
		{              
			status("Restoring server: asp_apache_$server->{asp_port}");
			$addserver = "./add_server \"asphome=$path\" \"non_interactive=yes\" start_asp=no start_apache=no";
		
			$addserver = "$addserver \"webserver_conf=$server->{webserver_conf}\"";
			$addserver = "$addserver \"start_asp_onboot=$server->{start_asp_onboot}\"";
			$addserver = "$addserver  \"asp_port=$server->{asp_port}\"";
			$addserver = "$addserver  \"webserver_binary=$server->{webserver_binary}\"";
				
			$ret = system($addserver);
			if ($ret)
			{
				fatal_error("Server configuration failed for server $server->{webserver_conf} - ret=$ret");
			}
			system("$path/INSTALL/mergeini srcini=$server->{config_files}/casp.cnfg dstini=$path/asp-apache-$server->{asp_port}/casp.cnfg");
			system("$path/INSTALL/mergeini srcini=$server->{config_files}/odbc.ini dstini=$path/asp-apache-$server->{asp_port}/odbc.ini");
			
			#
			# TODO - 
			#
			# now remove old datasources from odbc.ini
			#my $odbcini = INSTALL::ini::load("$path/asp-apache-$server->{asp_port}/odbc.ini");
			#my $odbcsec = INSTALL::ini::openSection($odbcini, "ODBC Data Sources");
			#if (!$odbcsec) {
			#	error("Couldn't open odbc.ini section - [ODBC Data Sources]");
			#}
			
			
		}
		else
		{
			status("Warning: section not found: asp_apache_$server->{asp_port}");  
		}
		$i += 1;
	}
	
	# install all components 
	# (should be done before server are started)
	#########################################################
	$i = 0;
	while ($component = INSTALL::ini::openSection($def, "component$i"))
	{
		status("Restoring component: $component->{name}");
		if (system($component->{postupgrade}))
		{
			error("Failed to post-upgrade component $component->{name}");
		}
	
		$i += 1;
	}
	
	# restart all web servers and asp servers
	#########################################################
	$server_count = $psec->{servers};
	$i = 0;
	while ($i < $server_count)
	{        
		$server = INSTALL::ini::openSection($def, "server$i");
		if ($server)
		{              
			if ("yes" eq $server->{start_apache})
			{
				my $installed_db = INSTALL::ini::open("$pGlobals->{installed_db}");
				my $section = INSTALL::ini::openSection($installed_db, "$path/asp-apache-$server->{asp_port}");
				if (!$section)
				{
					fatal_error("Could not find server ($path/asp-apache-$server->{asp_port}) in - $pGlobals->{installed_db}");
				}
	
				if (INSTALL::apache::restart_apache(%{$section}) == 0)
				{
					fatal_error("Failed to restart apache server");
				}
			}
	
			if ("yes" eq $server->{start_asp})
			{
				system("/etc/rc.d/init.d/asp-apache-$server->{asp_port} start");
			}
		}
		else
		{
			status("Warning: section not found: asp_apache_$server->{asp_port}");  
		}
		$i += 1;
	}
	
	
	chdir($pwd);

	return(0);
}

