//PhoneNumberVerifier.java

package AddressBook;


class PhoneNumberVerifier extends java.util.StringTokenizer
{
	private static final int STATE_START = 0;
	private static final int STATE_GOT_1 = 1;
	private static final int STATE_GOT_AREA_CODE = 2;
	private static final int STATE_GOT_PREFIX = 3;
	private static final int STATE_OK = 4;
	private static final int STATE_ERROR = 5;

	private String firsttoken;

	//we'll pull substrings off starting phonenumber with boundaries between 
	//substrings defined by "-" 's or spaces
	public PhoneNumberVerifier(String str)
	{
		super(str , "- ");
	}

	//main method which determines whether initial string is a valid phone
	//number
	public boolean Verify()
	{
		try
		{
			int state = STATE_START;

			while (state != STATE_OK && state != STATE_ERROR)
			{
				//get next substring and determine whether we still have
				//a valid phone number
				state = VerifyHelper(state);
			}

			if (state == STATE_OK)
			{
				//found last 4-digit string.  If we are at end of string
				//we found a good phone number
				try
				{
					String strNext = nextToken();
				
				}
				catch (java.util.NoSuchElementException e)
				{
					//no characters remaining in big string.  Normal condition
					return true;
				}

				//remaining characters after last 4-digit group
				return false;
			}
			else
			{
				return false;
			}
		}
		catch (BadPhoneNumberException e)
		{
			return false;
		}
	}

	private int VerifyHelper( int state) throws BadPhoneNumberException
	{
		//get next token.  Bail with STATE_ERROR if at end.
		String strNext;

		try
		{
			strNext = nextToken();
			if (firsttoken == null)
			{
				firsttoken = strNext;
			}
		}
		catch (java.util.NoSuchElementException e)
		{
			throw new BadPhoneNumberException();
		}
		
		//run state machine checking whether next token is expected in a
		//phone number format (either 1-xxx-xxx-xxxx or xxx-xxxx)
		try
		{
			switch (state)
			{
				case STATE_START:
					if (gotOne(strNext))
					{
						//first token is 1
						return STATE_GOT_1;
					}

					if (gotThree(strNext))
					{
						//if first token is not a 1 signifying long distance
						//it has to be a local prefix
						return STATE_GOT_AREA_CODE;
					}

					return STATE_ERROR;
					
				case STATE_GOT_1:

					if (gotThree(strNext))
					{
						return STATE_GOT_AREA_CODE;
					}

					//area code has to follow initial "1"
					return STATE_ERROR;


				case STATE_GOT_AREA_CODE:

					if (gotThree(strNext))
					{
						return STATE_GOT_PREFIX;
					}


					//maybe the area code was really a prefix
					if (! firsttoken.equals("1") && gotFour(strNext))
					{
						return STATE_OK;
					}

					//have to have local prefix after area code
					return STATE_ERROR;


				case STATE_GOT_PREFIX:

					if (gotFour(strNext))
					{
						//got complete phone number
						return STATE_OK;
					}

					//need 4 digits after prefix
					return STATE_ERROR;

				
				default:
					//can't get here
					throw new BadPhoneNumberException();
			}
		}
		catch (Exception e)
		{
			throw new BadPhoneNumberException();
		}
			
	}

	private boolean gotOne(String str)
	{
		if (str.equals ("1"))
		{
			return true;
		}

		return false;
	}

	private boolean gotThree(String str)
	{
		if (str.length() != 3)
		{
			return false;
		}

		try
		{
			int num = new Integer(str).intValue();
			if ( num < 100 || num > 999)
			{
				return false;
			}

			return true;
				
		}
		catch (NumberFormatException e)
		{
			return false;
		}
	}

	private boolean gotFour(String str)
	{
		if (str.length() != 4)
		{
			return false;
		}

		try
		{
			int num = new Integer(str).intValue();
			if ( num > 9999)
			{
				return false;
			}

			return true;
				
		}
		catch (NumberFormatException e)
		{
			return false;
		}
	}
}

