//Table.java

package DatabaseBase;

import java.io.*;
import java.util.Vector;

public class Table implements Serializable
{
	private Vector vect; 

	//global position member.  This implementation won't work well for
	//multiple users.  To make it work, need to implement sessions with
	//each user's Session object storing a Position.  Also need versions
	//of each method of this class that use the position member which get
	//passed Session arguments.  These methods would use the Position
	//stored in the Session objects instead of this global one
	public int Position = -1;

	//if we want the table to be persistent
	public String m_filename;

	//use this constructor for a non-persistent table
	public Table()
	{
		vect = new Vector();
	}

	//pass a writable filename to make table persistent
	public Table(String filename)
	{
		vect = new java.util.Vector();
		m_filename = filename;
	}

	public synchronized int size(){return vect.size();}

	//add record at end of table
	//sessionless version
	public synchronized void  addRecord(Record r)
	{
		vect.addElement(r);
		if (Position == -1)
		{
			Position = 0;
		}

	}


	public synchronized void insertRecord(Record r)
	{
		vect.insertElementAt(r, Position);
		Position++;
	}

	
	public synchronized void replaceRecord(Record r)
	{
		vect.setElementAt(r, Position);
	}


	//get record at current position
	public synchronized Record getRecord()
	{
		if (Position == -1 )
		{
			return null;
		}

		return (Record)vect.elementAt(Position);
	}



	public synchronized Record[] getRecords()
	{
		int iSize = size();
		Record[] records = new Record[iSize];
		for (int i = 0; i < iSize; i++)
		{
			records[i] = (Record)vect.elementAt(i);
		}

		return records;

	}

	public synchronized boolean EOF()
	{
		if (Position == vect.size() - 1)
		{
			return true;
		}

		return false;
	}

	

	public synchronized boolean BOF()
	{
		if (Position == -1 || Position == 0)
		{
			return true;
		}

		return false;
	}


 
	public synchronized boolean hasRecords()
	{
		if (Position == -1)
		{
			return false;
		}

		return true;
	}

	

	public synchronized void reset()
	{
		if (hasRecords())
		{
			Position = 0;
		}
		else
		{
			Position = -1;
		}
	
	}

	

	public synchronized void movePrevious()
	{
		if (!BOF())
		{
			Position --;
		}
	}

	

	public synchronized void moveNext()
	{
		if (!EOF())
		{
			Position++;
		}
	}



	public synchronized void  load() throws IOException, ClassNotFoundException
	{
		if (m_filename == null)
		{
			return;
		}

		FileInputStream fis;
		ObjectInputStream iis;

	
		fis = new FileInputStream(m_filename);
		iis = new ObjectInputStream(fis);
		

		Vector savedvect = (Vector)iis.readObject();
		if (savedvect != null)
		{
			vect = savedvect;
		}

		fis.close();
	}

	public synchronized void save() throws IOException, ClassNotFoundException
	{
		if (m_filename == null)
		{
			return;
		}

		FileOutputStream fos;
		ObjectOutputStream oos;
		
	
		fos = new FileOutputStream(m_filename);
		oos = new ObjectOutputStream(fos);
		

		oos.writeObject(vect);
		oos.flush();
		fos.close();
	}
}

