import java.awt.*;

//
//
// ColoredButton
//
//
public class ColoredButton extends Canvas
{
	protected String m_text = "";
	protected Color m_clrLight = Color.white;
	protected Color m_clrDark = Color.black;
	protected Color m_clrText = Color.black;
	protected boolean m_pressed = false;
	protected Rectangle m_rectText = null;

	protected static final int MARGIN = 2;
	protected static final int BEVEL = 2;

	protected int m_width;
	protected int m_height;

	public ColoredButton( String label, Color text, Color face, Color light, Color dark )
	{
		m_text = label;
		m_clrLight = light;
		m_clrDark = dark;
		m_clrText = text;
		setBackground( face );
	}

	public void paint(Graphics g)
	{
		int delta;
		if ( m_pressed )
			delta = 1;
		else
			delta = 0;

		drawShading( g );

		int width = size().width;
		FontMetrics fm = g.getFontMetrics();
		m_rectText = new Rectangle( 
			(width-fm.stringWidth(m_text))/2,
			BEVEL + MARGIN,
			fm.stringWidth(m_text) + 1,
			fm.getHeight() + 1 );

		g.setColor( m_clrText );
		g.drawString( m_text, 
			(width-fm.stringWidth(m_text))/2 + delta,
			BEVEL + MARGIN + fm.getMaxAscent() + delta );
	}

	protected void drawShading(Graphics g)
	{
		int width = size().width;
		int height = size().height;

		if ( m_pressed )
		{
			g.setColor( m_clrDark );
			topLeft( g, width, height );
			g.setColor( m_clrLight );
			bottomRight( g, width, height );
		}
		else
		{
			g.setColor( m_clrLight );
			topLeft( g, width, height );
			g.setColor( m_clrDark );
			bottomRight( g, width, height );
		}
	}

	protected void topLeft(Graphics g, int w, int h)
	{
		for( int ndx = 0; ndx < BEVEL; ++ndx )
		{
			g.drawLine( ndx, ndx, ndx, h - ndx - 1 );
			g.drawLine( ndx, ndx, w - ndx - 1, ndx );
		}
	}

	protected void bottomRight(Graphics g, int w, int h)
	{
		for( int ndx=0; ndx < BEVEL; ++ndx )
		{
			g.drawLine( w - ndx - 1, h - ndx - 1, ndx + 1, h - ndx - 1 );
			g.drawLine( w - ndx - 1, h - ndx - 1, w - ndx - 1, ndx + 1 );
		}
	}

	public Dimension preferredSize()
	{
		Font font = getFont();
		if ( font != null )
		{
			FontMetrics fm = getFontMetrics(font);
			return new Dimension(
				fm.stringWidth(m_text)+2*MARGIN+2*BEVEL,
				fm.getHeight()+2*MARGIN+2*BEVEL );
		}
		else
			return new Dimension(10,10);
	}

	public Dimension minimumSize()
	{
		Font font = getFont();
		if ( font != null )
		{
			FontMetrics fm = getFontMetrics(font);
			return new Dimension(
				fm.stringWidth(m_text)+2*MARGIN+2*BEVEL,
				fm.getHeight()+2*MARGIN+2*BEVEL );
		}
		else
			return new Dimension(10,10);
	}

	public boolean mouseDown(Event evt, int x, int y)
	{
		if ( !m_pressed )
		{
			m_pressed = true;
			changeState();
		}
		if ( m_text == null || m_text.equals("") )
			return false;
		return true;
	}

	public boolean mouseUp(Event evt, int x, int y )
	{
		if ( m_pressed )
		{
			m_pressed = false;
			changeState();
			// tell our parent we clicked!
			Event event = new Event( this, Event.ACTION_EVENT, this );
			getParent().postEvent( event );
		}
		return true;
	}

	public boolean mouseExit(Event evt, int x, int y)
	{
		if ( m_pressed )
		{
			m_pressed = false;
			changeState();
		}
		return true;
	}

	protected void changeState()
	{
		Graphics g = getGraphics();
		drawShading(g);
		if ( m_rectText != null )
		{
			repaint( m_rectText.x, m_rectText.y, 
				m_rectText.width, m_rectText.height );
		}
	}
}

