import java.awt.*;

//
//
// DisplayItem
//
//
public class DisplayItem 
{
	public int m_level = 0;
	public boolean m_visible;
	public int m_plusOrMinus;
	public String m_text;
	public String m_url;
	public String m_caption;
	public boolean m_selected;
	public Color m_color = null;

	protected Container m_parent = null;
	protected Rectangle m_rectPM = null;
	protected Rectangle m_rectLink = null;
	protected int m_baseline = 0;

	public final static int INDENT = 30;
	public final static int MARGIN = 0;

	public final static int NONE  = 0;
	public final static int PLUS  = 1;
	public final static int MINUS = 2;
	public final static int LINK  = 3;

	protected static Image s_plus = null;
	protected static Image s_minus = null;
	protected static int s_pmSize = 9;

	public DisplayItem( Container parent, int level, boolean visible, 
		Color color, String text, String url, String caption )
	{
		m_parent = parent;
		m_level = level;
		m_visible = visible;
		m_color = color;
		m_text = text;
		m_url = url;
		m_caption = caption;
		m_plusOrMinus = NONE;
		m_selected = false;
	}

	public int hitArea( int x, int y )
	{
		// check to see if the click was on the p/m box....
		if ( m_plusOrMinus != NONE )
		{
			if ( m_rectPM != null && m_rectPM.inside( x, y ) )
				return m_plusOrMinus;
			else if ( m_rectLink != null && m_rectLink.inside( x, y ) 
				&& m_url != null && !m_url.equals("") )
				return LINK;
		}
		return NONE;
	}

	public void draw( Graphics g, int yPos, int[] columns, Color clrBg )
	{
		// create the plus/minus boxes if they aren't already....
		if ( s_plus == null || s_minus == null )
			createPlusMinus();

		FontMetrics fm = g.getFontMetrics();
		m_baseline = yPos + MARGIN + fm.getMaxAscent();

		// shade the whole item background (if we want to)....
		if ( clrBg != null )
		{
			int width = columns[0] + columns[1] + columns[2] + columns[3];
			Rectangle rect = new Rectangle(0, yPos, 
				width, fm.getHeight() + 2*DisplayItem.MARGIN );
			Color oldColor = g.getColor();
			g.setColor( clrBg );
			g.fillRect( rect.x, rect.y, rect.width, rect.height );
			g.setColor( oldColor );
		}

		// figure the rectangle for the possible plus/minus box....
		int xPos = 10 + m_level*INDENT;
		m_rectPM = new Rectangle( xPos, 
			yPos + MARGIN + (fm.getMaxAscent() - s_pmSize), 
			s_pmSize, s_pmSize );

		// draw the plus or minus box....
		if ( m_plusOrMinus == PLUS && s_plus != null )
		{
			g.drawImage( s_plus, m_rectPM.x, m_rectPM.y, m_parent );
			xPos += s_pmSize + 10;
		}
		else if (m_plusOrMinus == MINUS && s_minus != null)
		{
			g.drawImage( s_minus, m_rectPM.x, m_rectPM.y, m_parent );
			xPos += s_pmSize + 10;
		}
	
		// calculate the click box for the link....
		m_rectLink = new Rectangle(xPos, yPos + MARGIN,
			fm.stringWidth(m_text), fm.getHeight() );

		Color oldColor = null;
		if ( m_selected )
		{
			oldColor = g.getColor();
			g.setColor( m_color );
		}
		if ( m_url != null && !m_url.equals("") )
		{
			g.drawLine( m_rectLink.x, m_baseline + 1,
				m_rectLink.x + m_rectLink.width, m_baseline + 1);
		}

		g.drawString( m_text, xPos, m_baseline );
		if ( oldColor != null )
			g.setColor( oldColor );
}

	private void createPlusMinus()
	{
		// Create Images
		if ( s_plus == null )
			s_plus = m_parent.createImage(9, 9);
		if ( s_minus == null )
			s_minus = m_parent.createImage(9, 9);

		s_pmSize = s_plus.getWidth(m_parent);

		// Draw plus
		Graphics g = s_plus.getGraphics();
		g.setColor(Color.white);
		g.fillRect( 0, 0, s_pmSize, s_pmSize );
		g.setColor(Color.gray);
		g.drawRect(0, 0, 8, 8);
		g.setColor(Color.black);
		g.drawLine(2, 4, 6, 4);
		g.drawLine(4, 2, 4, 6);

		// Draw minus
		g = s_minus.getGraphics();
		g.setColor(Color.white);
		g.fillRect( 0, 0, s_pmSize, s_pmSize );
		g.setColor(Color.gray);
		g.drawRect(0, 0, 8, 8);
		g.setColor(Color.black);
		g.drawLine(2, 4, 6, 4);
	}
}

