//******************************************************************************
// Policies.java:	Applet
//
//******************************************************************************
import java.applet.*;
import java.awt.*;
import java.util.*;
import java.net.*;
import java.io.*;

//==============================================================================
// Main Class for applet Policies
//
//==============================================================================
public class Policies extends Applet
{
	private ColoredButton		m_hdrPolicy = null;
	private ColoredButton		m_hdrEffDate = null;
	private ColoredButton		m_hdrExpDate = null;
	private ColoredButton		m_hdrType = null;
	private TableTree			m_table = null;

	public Scrollbar			m_scroll = null;

	// PARAMETER SUPPORT:
	//		Parameters allow an HTML author to pass information to the applet;
	// the HTML author specifies them using the <PARAM> tag within the <APPLET>
	// tag.  The following variables are used to store the values of the
	// parameters.
    //--------------------------------------------------------------------------

    // Members for applet parameters
    // <type>       <MemberVar>    = <Default Value>
    //--------------------------------------------------------------------------
	private URL		m_urlPolicies = null;
	public Color	m_clrBg = Color.white;
	public Color	m_clrStripe = null;
	public Color	m_clrLink = new Color( 0x33, 0x66, 0xcc );
	public Color	m_clrPending = new Color( 0xe7, 0x9c, 0x84 );
	public Color	m_hdrLight = Color.white;
	public Color	m_hdrDark = Color.darkGray;
	public Color	m_hdrFace = Color.lightGray;
	public Color	m_hdrText = Color.black;
	public int		m_fontSize = 14;
	public String	m_target = "_top";

    // Parameter names.  To change a name of a parameter, you need only make
	// a single change.  Simply modify the value of the parameter string below.
    //--------------------------------------------------------------------------
	private final String PARAM_Policies = "Policies";
	private final String PARAM_Color = "Color";
	private final String PARAM_ClrStripe = "StripeColor";
	private final String PARAM_ClrLink = "LinkColor";
	private final String PARAM_ClrPending = "PendingColor";
	private final String PARAM_HdrLight = "HeaderHighlight";
	private final String PARAM_HdrDark = "HeaderShadow";
	private final String PARAM_HdrFace = "HeaderFace";
	private final String PARAM_HdrText = "HeaderText";
	private final String PARAM_FontSize = "FontSize";
	private final String PARAM_TargetFrm = "TargetFrame";

	// Policies Class Constructor
	//--------------------------------------------------------------------------
	public Policies()
	{
	}

	// APPLET INFO SUPPORT:
	//		The getAppletInfo() method returns a string describing the applet's
	// author, copyright date, or miscellaneous information.
    //--------------------------------------------------------------------------
	public String getAppletInfo()
	{
		return "Name: Policies\r\n" +
		       "Author: Ron Logan\r\n" +
		       "Created with Microsoft Visual J++ Version 1.1";
	}

	// PARAMETER SUPPORT
	//		The getParameterInfo() method returns an array of strings describing
	// the parameters understood by this applet.
	//
    // Policies Parameter Information:
    //  { "Name", "Type", "Description" },
    //--------------------------------------------------------------------------
	public String[][] getParameterInfo()
	{
		String[][] info =
		{
			{ PARAM_Policies, "Relative or absolute URL", "URL of Policy information" },
			{ PARAM_Color, "hexadecimal color value", "Background color" },
			{ PARAM_ClrStripe, "hexadecimal color value", "Background color of striped items" },
			{ PARAM_FontSize, "Integer", "Font size on points" },
			{ PARAM_ClrLink, "hexadecimal color value", "Color of links" },
			{ PARAM_HdrLight, "hexadecimal color value", "Color of header highlights" },
			{ PARAM_HdrDark, "hexadecimal color value", "Color of header shadows" },
			{ PARAM_HdrFace, "hexadecimal color value", "Color of header face" },
			{ PARAM_HdrText, "hexadecimal color value", "Color of header text" },
			{ PARAM_ClrPending, "hexadecimal color value", "Color of pending policy items" },
			{ PARAM_TargetFrm, "String frame name", "target frame for links" },
		};
		return info;		
	}

	protected void parseParams()
	{
		// PARAMETER SUPPORT
		//		The following code retrieves the value of each parameter
		// specified with the <PARAM> tag and stores it in a member
		// variable.
		//----------------------------------------------------------------------
		String param;

		// Policy: Parameter description
		//----------------------------------------------------------------------
		param = getParameter(PARAM_Policies);
		if (param != null)
		{
			try
			{
				m_urlPolicies = new URL( param );
			}
			catch(MalformedURLException ex1)
			{
				try
				{
					m_urlPolicies = new URL( getDocumentBase(), param );
				}
				catch(MalformedURLException ex2)
				{
					m_urlPolicies = null;
				}
			}
		}
		param = getParameter(PARAM_Color);
		if ( param != null )
		{
			try
			{
				// first see if we've got a hex number....
				int nColor = Integer.parseInt( param, 16 );
				m_clrBg = new Color( nColor );
			}
			catch(NumberFormatException ex)
			{
				// default...
				m_clrBg = Color.white;
			}
		}
		param = getParameter(PARAM_ClrStripe);
		if ( param != null )
		{
			try
			{
				// first see if we've got a hex number....
				int nColor = Integer.parseInt( param, 16 );
				m_clrStripe = new Color( nColor );
			}
			catch(NumberFormatException ex)
			{
				// default...
				m_clrStripe = null;
			}
		}
		param = getParameter(PARAM_ClrLink);
		if ( param != null )
		{
			try
			{
				// first see if we've got a hex number....
				int nColor = Integer.parseInt( param, 16 );
				m_clrLink = new Color( nColor );
			}
			catch(NumberFormatException ex)
			{
				// default...
				m_clrLink = new Color( 0x33, 0x66, 0xcc );
			}
		}
		param = getParameter(PARAM_ClrPending);
		if ( param != null )
		{
			try
			{
				// first see if we've got a hex number....
				int nColor = Integer.parseInt( param, 16 );
				m_clrPending = new Color( nColor );
			}
			catch(NumberFormatException ex)
			{
				// default...
				m_clrPending = new Color( 0xe7, 0x9c, 0x84 );
			}
		}
		param = getParameter(PARAM_HdrLight);
		if ( param != null )
		{
			try
			{
				// first see if we've got a hex number....
				int nColor = Integer.parseInt( param, 16 );
				m_hdrLight = new Color( nColor );
			}
			catch(NumberFormatException ex)
			{
				// default...
				m_hdrLight = Color.white;
			}
		}
		param = getParameter(PARAM_HdrDark);
		if ( param != null )
		{
			try
			{
				// first see if we've got a hex number....
				int nColor = Integer.parseInt( param, 16 );
				m_hdrDark = new Color( nColor );
			}
			catch(NumberFormatException ex)
			{
				// default...
				m_hdrDark = Color.darkGray;
			}
		}
		param = getParameter(PARAM_HdrFace);
		if ( param != null )
		{
			try
			{
				// first see if we've got a hex number....
				int nColor = Integer.parseInt( param, 16 );
				m_hdrFace = new Color( nColor );
			}
			catch(NumberFormatException ex)
			{
				// default...
				m_hdrFace = Color.lightGray;
			}
		}
		param = getParameter(PARAM_HdrText);
		if ( param != null )
		{
			try
			{
				// first see if we've got a hex number....
				int nColor = Integer.parseInt( param, 16 );
				m_hdrText = new Color( nColor );
			}
			catch(NumberFormatException ex)
			{
				// default...
				m_hdrText = Color.black;
			}
		}
		param = getParameter(PARAM_FontSize);
		if ( param != null )
		{
			try
			{
				m_fontSize = Integer.parseInt( param, 10 );
			}
			catch(NumberFormatException ex)
			{
				m_fontSize = 14;
			}
		}
		param = getParameter(PARAM_TargetFrm);
		if ( param != null )
			m_target = param;
	}

	// The init() method is called by the AWT when an applet is first loaded or
	// reloaded.  Override this method to perform whatever initialization your
	// applet needs, such as initializing data structures, loading images or
	// fonts, creating frame windows, setting the layout manager, or adding UI
	// components.
    //--------------------------------------------------------------------------
	public void init()
	{
		// parse our applet parameters....
		parseParams();

        // If you use a ResourceWizard-generated "control creator" class to
        // arrange controls in your applet, you may want to call its
        // CreateControls() method from within this method. Remove the following
        // call to resize() before adding the call to CreateControls();
        // CreateControls() does its own resizing.
        //----------------------------------------------------------------------
		setBackground( m_clrBg );
		setFont( new Font( "MS Sans Serif", Font.PLAIN, m_fontSize ) );
		setLayout( new GridBagLayout() );

		Font fntButton = new Font( "MS Sans Serif", Font.BOLD, m_fontSize*6/7 );
		m_hdrPolicy = new ColoredButton( "Policy No.", m_hdrText, m_hdrFace, m_hdrLight, m_hdrDark );
		m_hdrPolicy.setFont( fntButton );

		m_hdrType = new ColoredButton( "Type", m_hdrText, m_hdrFace, m_hdrLight, m_hdrDark );
		m_hdrType.setFont( fntButton );
		
		m_hdrEffDate = new ColoredButton( "Effective", m_hdrText, m_hdrFace, m_hdrLight, m_hdrDark );
		m_hdrEffDate.setFont( fntButton );
		
		m_hdrExpDate = new ColoredButton( "Expires", m_hdrText, m_hdrFace, m_hdrLight, m_hdrDark );
		m_hdrExpDate.setFont( fntButton );

		AddButton( m_hdrPolicy, 0, 1.0 );
		AddButton( m_hdrType, 1, 0.1 );
		AddButton( m_hdrEffDate, 2, 0.2 );
		AddButton( m_hdrExpDate, 3, 0.2 );

		m_scroll = new Scrollbar( Scrollbar.VERTICAL );
		m_scroll.setBackground( Color.lightGray );
		m_scroll.setValues( 0, 0, 0, 0 );
		AddElement( m_scroll, 4, 1, 1, 1, 0.0, 0.0, GridBagConstraints.VERTICAL );

		m_table = new TableTree( this, m_urlPolicies );
		AddElement( m_table, 0, 1, 4, 1, 1.0, 1.0, GridBagConstraints.BOTH );
	}

	void AddButton( Component comp, int pos, double weightx ) 
	{
		AddElement( comp, pos, 0, 1, 1, weightx, 0.0,
			GridBagConstraints.HORIZONTAL );
	}

	void AddElement( Component comp, int x, int y, int dx, int dy,
		double weightx, double weighty, int fill )
	{
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = dx;
		gbc.gridheight = dy;
		gbc.weightx = weightx;
		gbc.weighty = weighty;
		gbc.anchor = GridBagConstraints.NORTH;
		gbc.fill = fill;

		((GridBagLayout)this.getLayout()).setConstraints( comp, gbc );
		this.add( comp );
	}

	public boolean handleEvent( Event event )
	{
		switch( event.id )
		{
		case Event.ACTION_EVENT:
			if ( event.target == m_hdrPolicy )
				m_table.Sort( TableTree.ON_POLICY );
			else if ( event.target == m_hdrEffDate )
				m_table.Sort( TableTree.ON_EFFDATE );
			else if ( event.target == m_hdrExpDate )
				m_table.Sort( TableTree.ON_EXPDATE );
			else if ( event.target == m_hdrType )
				m_table.Sort( TableTree.ON_TYPE );
			return true;
		case Event.SCROLL_ABSOLUTE:
			m_table.scrollTo( m_scroll.getValue() );
			return true;
		}
		return super.handleEvent( event );
	}

	// Place additional applet clean up code here.  destroy() is called when
	// when you applet is terminating and being unloaded.
	//-------------------------------------------------------------------------
	public void destroy()
	{
		// TODO: Place applet cleanup code here
	}

	// Policies Paint Handler
	//--------------------------------------------------------------------------
	public void paint(Graphics g)
	{
	}

	//		The start() method is called when the page containing the applet
	// first appears on the screen. The AppletWizard's initial implementation
	// of this method starts execution of the applet's thread.
	//--------------------------------------------------------------------------
	public void start()
	{
	}
	
	// The stop() method is called when the page containing the applet is
	// no longer on the screen. The AppletWizard's initial implementation of
	// this method stops execution of the applet's thread.
	//--------------------------------------------------------------------------
	public void stop()
	{
	}
}

