//
//
// TableTree
//
//
import java.applet.*;
import java.awt.*;
import java.util.*;
import java.net.*;
import java.io.*;

class TableTree extends Panel
{
	Policies m_policies = null;
	Vector m_list = new Vector( 5, 5 );

	int m_topItem = 0;
	int m_botItem = 0;
	int m_lineHeight = 18;

	public static final int ON_POLICY = 0;
	public static final int ON_EFFDATE = 1;
	public static final int ON_EXPDATE = 2;
	public static final int ON_TYPE = 3;

	protected DisplayItem m_selected = null;

	TableTree( Policies pols, URL url ) 
	{
		m_policies = pols;
		if ( url != null )
			loadFromURL( url );
	}

	protected boolean loadFromURL( URL urlSource )
	{
		try {
			DataInputStream dis = new DataInputStream(urlSource.openStream());
			m_policies.showStatus( "Loading policy information...." );	

			int level;
			int lastLevel = 0;
			String itemLine;
			StringTokenizer tokenizer;
			DisplayItem item = null;
			while ((itemLine = dis.readLine()) != null) 
			{
				tokenizer = new StringTokenizer(itemLine, ";");
				if (tokenizer.hasMoreTokens()) 
				{
					try
					{
						level = Integer.parseInt( tokenizer.nextToken() );
						if ( item != null && level > lastLevel )
							item.m_plusOrMinus = DisplayItem.PLUS;
						lastLevel = level;

						String name = "";
						String type = "";
						String url = null;
						String caption = null;
						Date effDate = null;
						Date expDate = null;
						Color color = null;
						boolean approved = false;
						switch( level )
						{
						case 0:
							if ( tokenizer.hasMoreElements() )
								name = tokenizer.nextToken().trim();
							if ( tokenizer.hasMoreElements() )
								type = tokenizer.nextToken().trim();
							if ( tokenizer.hasMoreElements() )
								effDate = new Date( tokenizer.nextToken() );
							if ( tokenizer.hasMoreElements() )
								expDate = new Date( tokenizer.nextToken() );
							if ( tokenizer.hasMoreElements() )
								url = tokenizer.nextToken().trim();
							if ( tokenizer.hasMoreElements() )
								caption = tokenizer.nextToken().trim();
							item = new PolicyItem( this, name, type, effDate, expDate, 
								m_policies.m_clrLink, url, caption );
							break;
						case 1:
							if ( tokenizer.hasMoreElements() )
								name = tokenizer.nextToken().trim();
							if ( tokenizer.hasMoreElements() )
							{
								if ( tokenizer.nextToken().trim().equalsIgnoreCase( "Y" ) )
									approved = true;
								else
								{
									approved = false;
									color = m_policies.m_clrPending;
								}
							}
							if ( tokenizer.hasMoreElements() )
								url = tokenizer.nextToken().trim();
							if ( tokenizer.hasMoreElements() )
								caption = tokenizer.nextToken().trim();
							item = new TextItem( this, level, name, approved, color, url, caption );
							break;
						default:
							if ( tokenizer.hasMoreElements() )
								name = tokenizer.nextToken().trim();
							if ( tokenizer.hasMoreElements() )
								url = tokenizer.nextToken().trim();
							if ( tokenizer.hasMoreElements() )
								caption = tokenizer.nextToken().trim();
							item = new TextItem( this, level, name, true, color, url, caption );
							break;
						}
						m_list.addElement( item );
					}
					catch( NumberFormatException ex )
					{
						// ignore lines that don't start with a level!
						level = 0;
					}
				}
			}
			m_policies.showStatus( "Done." );
		}
		catch (IOException ex)
		{
			m_policies.showStatus( "Error reading policy information!" );
		}

		return true;
	}

	public void paint( Graphics g )
	{
		GridBagLayout gridbag = ((GridBagLayout)m_policies.getLayout());
		int[] columns = (gridbag.getLayoutDimensions())[0];

		int numViewable = 0;
		int numVisible = 0;
		int numItems = m_list.size();

		FontMetrics fm = g.getFontMetrics();
		m_lineHeight = fm.getHeight() + 2*DisplayItem.MARGIN;

		Dimension dim = size();
		int numLines = dim.height/m_lineHeight;
		int lastVisible = 0;
		int yPos = 0;

		Color bgColor = null;

		m_botItem = numItems;
		for( int ndx = 0; ndx < numItems; ++ndx )
		{
			DisplayItem item = (DisplayItem)(m_list.elementAt(ndx));
			// toggle the stripe color (if any)....
			if ( item.m_level == 0 && m_policies.m_clrStripe != null )
			{
				if ( bgColor == m_policies.m_clrBg )
					bgColor = m_policies.m_clrStripe;
				else
					bgColor = m_policies.m_clrBg;
			}
			// if we should paint it, do it!
			if ( item.m_visible && item.m_level-1 <= lastVisible )
			{
				++numViewable;
				lastVisible = item.m_level;
				if ( m_topItem < numViewable && numVisible < numLines )
				{
					++numVisible;
					item.draw( g, yPos, columns, bgColor );
					yPos += m_lineHeight;
					m_botItem = ndx;
				}
			}
		}

		Scrollbar sb = m_policies.m_scroll;
		if ( numVisible < numViewable )
		{
			// we wanna see the scrollbar....
			sb.setPageIncrement( numLines-1 );
			sb.setValues( m_topItem, 0, 0, numViewable-numVisible );
			sb.show();
		}
		else
			sb.hide();
	}

	public void Sort( int onField )
	{
		Vector newList = new Vector( 5, 5 );
		Enumeration source = m_list.elements();
		int ndx = 0;
		while( source.hasMoreElements() )
		{
			DisplayItem item = (DisplayItem)(source.nextElement());
			if ( item instanceof PolicyItem )
			{
				PolicyItem pol = (PolicyItem)item;
				Enumeration dest = newList.elements();
				ndx = 0;
				while( dest.hasMoreElements() )
				{
					DisplayItem other = (DisplayItem)(dest.nextElement());
					if ( other instanceof PolicyItem )
					{
						PolicyItem pol2 = (PolicyItem)other;
						if ( onField == ON_EXPDATE && 
							pol.m_expires.before( pol2.m_expires ) )
							break;
						else if ( onField == ON_EFFDATE &&
							pol.m_effective.before( pol2.m_effective ) )
							break;
						else if ( onField == ON_TYPE &&
							pol.m_type.compareTo(pol2.m_type) <= 0 )
							break;
						else if ( onField == ON_POLICY &&
							pol.m_text.compareTo(pol2.m_text) <= 0 )
							break;
					}
					++ndx;
				}
			}
			newList.insertElementAt( item, ndx++ );
		}
		m_list = newList;
		repaint();
	}

	protected void selectItem( DisplayItem item )
	{
		Rectangle rect = null;
		if ( item == null && m_selected != null )
		{
			rect = m_selected.m_rectLink;
			m_selected.m_selected = false;
			m_selected = null;
			m_policies.showStatus( "" );
		}
		else if ( item != null )
		{
			rect = item.m_rectLink;
			m_selected = item;
			item.m_selected = true;
			if ( item.m_caption != null && !item.m_caption.equals("") )
				m_policies.showStatus( item.m_caption );
			else if ( item.m_url != null )
				m_policies.showStatus( item.m_url );
		}
		if ( rect != null )
			repaint(100, rect.x, rect.y, rect.width, rect.height);
	}

	// MOUSE SUPPORT
	public boolean mouseMove(Event evt, int x, int y)
	{
		if ( m_selected != null )
		{
			if ( m_selected.hitArea( x, y ) != DisplayItem.LINK )
			{
				selectItem( null );
				// recurse in case we entered another area....
				mouseMove( evt, x, y );
			}
		}
		else
		{
			DisplayItem item = itemAt( x, y );
			if ( item != null && item.hitArea(x,y) == DisplayItem.LINK )
				selectItem( item );
		}
		return true;
	}

	public boolean mouseExit(Event evt, int x, int y )
	{
		if ( m_selected != null )
			selectItem( null );
		return true;
	}

	public boolean mouseDown(Event evt, int x, int y)
	{
		DisplayItem item = itemAt( x, y );
		// single click....
		if ( item != null )
		{
			int action = item.hitArea( x, y );
			switch( action )
			{
			case DisplayItem.PLUS:
				item.m_plusOrMinus = DisplayItem.MINUS;
				showSubItems( item, true );
				break;
			case DisplayItem.MINUS:
				item.m_plusOrMinus = DisplayItem.PLUS;
				showSubItems( item, false );
				break;
			case DisplayItem.LINK:
				linkTo( item.m_url );
				break;
			case DisplayItem.NONE:
				break;
			}
		}
		return true;
	}

	protected void showSubItems( DisplayItem item, boolean visible )
	{
		int ndx = m_list.lastIndexOf( item );
		int cItems = m_list.size();
		while( ++ndx < cItems )
		{
			DisplayItem subItem = (DisplayItem)(m_list.elementAt(ndx));
			if ( subItem.m_level == item.m_level )
				break;
			else if ( subItem.m_level == (item.m_level + 1) )
				subItem.m_visible = visible;
		}
		repaint();
	}

	protected void linkTo( String strUrl )
	{
		if ( !strUrl.equals("") )
		{
			URL url = null;
			try{url = new URL( strUrl );}
			catch(MalformedURLException ex1)
			{
				try{url = new URL( m_policies.getDocumentBase(),strUrl );}
				catch(MalformedURLException ex2)
				{
					url = null;
				}
			}
			if ( url != null )
			{
				AppletContext ctx = m_policies.getAppletContext();
				ctx.showDocument( url, m_policies.m_target );
			}
		}
	}

	protected DisplayItem itemAt( int x, int y ) 
	{
		int line = (y-3)/m_lineHeight + m_topItem; // line number
		int visLine = 0;
		int numLines = m_list.size();
		int lastVisible = 0;
		for( int ndx=0; ndx <= numLines; ++ndx )
		{
			DisplayItem item = (DisplayItem)(m_list.elementAt(ndx));
			if ( item.m_visible && item.m_level-1 <= lastVisible )
			{
				lastVisible = item.m_level;
				if ( visLine++ == line )
					return item;
			}
		}
		return null;
	}

	protected void scrollTo( int value ) 
	{
		if ( m_topItem != value )
		{
			m_topItem = value;
			repaint();
		}
	}
}

