// AmSch.cpp : Implementation of CAmSch
#include "stdafx.h"
#include "calc.h"
#include <math.h>
#include "AmSch.h"

/////////////////////////////////////////////////////////////////////////////
// CAmSch

STDMETHODIMP CAmSch::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IAmSch,
	};
	for (int i=0;i<sizeof(arr)/sizeof(arr[0]);i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CAmSch::OnStartPage (IUnknown* pUnk)  
{
	if(!pUnk)
		return E_POINTER;

	CComPtr<IScriptingContext> spContext;
	HRESULT hr;

	// Get the IScriptingContext Interface
	hr = pUnk->QueryInterface(IID_IScriptingContext, (void **)&spContext);
	if(FAILED(hr))
		return hr;

	// Get Response Object Pointer
	hr = spContext->get_Response(&m_piResponse);
	if(FAILED(hr))
	{
		return hr;
	}
	
	
	m_bOnStartPageCalled = TRUE;
	return S_OK;
}

STDMETHODIMP CAmSch::OnEndPage ()  
{
	m_bOnStartPageCalled = FALSE;
	// Release all interfaces
	m_piResponse.Release();

	return S_OK;
}


STDMETHODIMP CAmSch::Calculate()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	double J, Q, C, H, M, N;
	int nCount;
	CString strOut;

	J = m_lRate / (1200);
    N = m_lLength * 12;
    nCount = 0;
    
    // Calculate the monthly payment
    M = m_lAmount * (J / (1 - pow((1 + J), -N)));

	// start table
	strOut.Format( "<table %s><tr><td %s>Payment</td><td %s>Principal</td>"
				   "<td %s>Interest</td><td %s>Balance</td></tr>", m_strTable, m_strCell,
				   m_strCell, m_strCell, m_strCell );

    m_piResponse->Write( COleVariant( strOut ) );

	do
	{
        // Loop until the balance is zero
        nCount++;
        H = m_lAmount * J;
        C = M - H;
        Q = m_lAmount - C;
        m_lAmount = Q;
		strOut.Format( "<tr><td %s>%i</td><td %s>%3.2f</td>"
					   "<td %s>%3.2f</td><td %s>$%3.2f</td></tr>", 
					   m_strCell, nCount, m_strCell, C, m_strCell,
				       H, m_strCell, Q );

        m_piResponse->Write( COleVariant( strOut ) );
	}
	while( Q >= 0 );
    
    // finish table
	m_piResponse->Write( COleVariant( "</table>" ) );

	return S_OK;
}

STDMETHODIMP CAmSch::put_InitialAmount(long newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_lAmount = newVal;

	return S_OK;
}

STDMETHODIMP CAmSch::put_InterestRate(long newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if( newVal < 30 )
	{
		m_lRate = newVal;
		return S_OK;
	}
	else
	{
		m_lRate = 30;
		return S_FALSE;
	}
}

STDMETHODIMP CAmSch::put_LengthInYears(long newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_lLength = newVal;

	return S_OK;
}

STDMETHODIMP CAmSch::put_TableFormat(BSTR newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_strTable = newVal;

	return S_OK;
}

STDMETHODIMP CAmSch::put_CellFormat(BSTR newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_strCell = newVal;

	return S_OK;
}
