use INSTALL::common;

use INSTALL::global;
use INSTALL::parameter;
use INSTALL::os;

umask 0;

my $CONST_Package = "install.pl - admin-console - main";
my $CONST_ComponentName = "Administration Console";

STDOUT->autoflush(1);

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Internal functions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#------------------------------------------------------------------------------
# All of the error reporting mechanisms take of a list of strings, and format
# them such that they fall on different lines.
#------------------------------------------------------------------------------
sub error {
  my $result = INSTALL::common::generate_error(@_);
  print STDERR "Package location: $CONST_Package\n";
  return($result);
}

sub trace
{
  if ($debug)
  {
    my $result = INSTALL::common::generate_report("trace: ",@_);
    print STDERR "Package location: $CONST_Package\n";
    return($result);
  }
}

sub report 
{
  my $result = INSTALL::common::generate_report("",@_);
  return($result);
}

sub note
{
  my $result = INSTALL::common::generate_note(@_);
  return($result);
}

my $pParameters = \%INSTALL::parameter::variables;

#==============================================================================
#------------------------ Default global values
#==============================================================================

INSTALL::common::set_standard_globals;

my $pGlobals = \%INSTALL::global::variables;
my $pOS = \%INSTALL::os::Details;
my $interactive = 
  (not INSTALL::conditional::boolean_value($pParameters->{non_interactive}));

my $CONST_Uninstall = INSTALL::conditional::boolean_value($pParameters->{uninstall});

########## Adjust for per component values.

$pGlobals->{admin_dir} = $pParameters->{component_dir};
$pGlobals->{template_dir} = 
  $pGlobals->{templates_dir} = "$pParameters->{component_dir}/INSTALL/templates";
$pGlobals->{settings_reg} = "$pParameters->{component_dir}/INSTALL/settings.reg";

#==============================================================================
#------------------ Substitute settings.reg file for admin-console.
#==============================================================================

use INSTALL::substitute;

if (not INSTALL::substitute::template("settings.reg","$pParameters->{component_dir}/INSTALL"))
{
  error("Unable to generate settings.reg file in $pParameters->{component_dir}/INSTALL");
  exit(1);
}

#==============================================================================
#------------------------ Handle existing admin-console.
#==============================================================================

use INSTALL::component;
use INSTALL::query;
use INSTALL::admin;

my $pComponent = INSTALL::component::open($CONST_ComponentName,
					  $pGlobals->{asphome});
if ($pComponent && (not $CONST_Uninstall)) {
  if ($interactive) {
    print "\n";
    INSTALL::query::user_menu
      ("EXISTING ".uc($CONST_ComponentName)." DETECTED",
       "I have detected an existing $CONST_ComponentName associated with this ".
       "installation.  It has been configured to be running on port $pComponent->{admd_port}.  ".
       "I cannot install / upgrade this package until the old version has been removed."
      );

    my $answer = INSTALL::query::user
      ("Would you like to remove this $CONST_ComponentName on port $pComponent->{admd_port}",
       "terminal_mark=?","y","default=n");
    
    if ($answer eq "n") {
      report("Canceled installation of the $CONST_ComponentName component.");
      exit(1);
    }
  }
  else {
    ## Default to destruction in non_interactive.
  }

  unless (INSTALL::admin::uninstall) {
    error("Unable to successfully uninstall the $CONST_ComponentName component.");
  }
}
#==============================================================================
#------------------------ Confirm perl binary location.
#==============================================================================

use INSTALL::os;

# perl needs to be searched for and saved in a preceeding script.
sub confirm_perl_location 
{
  ($#_ == 0) || die "Invalid number of arguments to confirm_perl_location.";

  return(-x $_[0]);
}

$pGlobals->{perl} = $pOS->{perl};

if (not confirm_perl_location($pGlobals->{perl})) {
  if ($interactive) {
    $INSTALL::global::variables{perl} = INSTALL::query::user_function
      ("I was unable to locate your perl binary, enter it below\n",
       *confirm_perl_location);
  }
  else {
    error("Unable to discover location of perl binary.");
    exit(1);
  }
}


#==============================================================================
#------------------------ Configure the administration console.
#==============================================================================
use INSTALL::conditional;

my $result;

if ($CONST_Uninstall) {
  $result = INSTALL::admin::uninstall;
}
else {
  $result = INSTALL::admin::configure;
}

if ($pGlobals->{status_text} && 
    open(COMPONENT_LOG,">>$pGlobals->{component_log}")) 
{
  print COMPONENT_LOG $pGlobals->{status_text};
  close(COMPONENT_LOG);
}

exit(not $result);
